
From: "Antonino A. Daplas" <adaplas@gmail.com>

If nvidiafb fails to probe the EDID block, get the EDID from the BIOS.

Signed-off-by: Antonino Daplas <adaplas@pol.net>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 drivers/video/nvidia/nv_i2c.c   |   16 ++++++++++++----
 drivers/video/nvidia/nv_proto.h |    2 +-
 drivers/video/nvidia/nv_setup.c |    6 +++---
 3 files changed, 16 insertions(+), 8 deletions(-)

diff -puN drivers/video/nvidia/nv_i2c.c~nvidiafb-fallback-to-firmware-edid drivers/video/nvidia/nv_i2c.c
--- devel/drivers/video/nvidia/nv_i2c.c~nvidiafb-fallback-to-firmware-edid	2005-08-21 21:53:55.000000000 -0700
+++ devel-akpm/drivers/video/nvidia/nv_i2c.c	2005-08-21 21:53:55.000000000 -0700
@@ -194,8 +194,9 @@ static u8 *nvidia_do_probe_i2c_edid(stru
 	return NULL;
 }
 
-int nvidia_probe_i2c_connector(struct nvidia_par *par, int conn, u8 **out_edid)
+int nvidia_probe_i2c_connector(struct fb_info *info, int conn, u8 **out_edid)
 {
+	struct nvidia_par *par = info->par;
 	u8 *edid = NULL;
 	int i;
 
@@ -205,10 +206,17 @@ int nvidia_probe_i2c_connector(struct nv
 		if (edid)
 			break;
 	}
+
+	if (!edid && conn == 1) {
+		/* try to get from firmware */
+		edid = kmalloc(EDID_LENGTH, GFP_KERNEL);
+		if (edid)
+			memcpy(edid, fb_firmware_edid(info->device),
+			       EDID_LENGTH);
+	}
+
 	if (out_edid)
 		*out_edid = edid;
-	if (!edid)
-		return 1;
 
-	return 0;
+	return (edid) ? 0 : 1;
 }
diff -puN drivers/video/nvidia/nv_proto.h~nvidiafb-fallback-to-firmware-edid drivers/video/nvidia/nv_proto.h
--- devel/drivers/video/nvidia/nv_proto.h~nvidiafb-fallback-to-firmware-edid	2005-08-21 21:53:55.000000000 -0700
+++ devel-akpm/drivers/video/nvidia/nv_proto.h	2005-08-21 21:53:55.000000000 -0700
@@ -34,7 +34,7 @@ void NVLockUnlock(struct nvidia_par *par
 #if defined(CONFIG_FB_NVIDIA_I2C) || defined (CONFIG_PPC_OF)
 void nvidia_create_i2c_busses(struct nvidia_par *par);
 void nvidia_delete_i2c_busses(struct nvidia_par *par);
-int nvidia_probe_i2c_connector(struct nvidia_par *par, int conn,
+int nvidia_probe_i2c_connector(struct fb_info *info, int conn,
 			       u8 ** out_edid);
 #else
 #define nvidia_create_i2c_busses(...)
diff -puN drivers/video/nvidia/nv_setup.c~nvidiafb-fallback-to-firmware-edid drivers/video/nvidia/nv_setup.c
--- devel/drivers/video/nvidia/nv_setup.c~nvidiafb-fallback-to-firmware-edid	2005-08-21 21:53:55.000000000 -0700
+++ devel-akpm/drivers/video/nvidia/nv_setup.c	2005-08-21 21:53:55.000000000 -0700
@@ -401,7 +401,7 @@ void NVCommonSetup(struct fb_info *info)
 	nvidia_create_i2c_busses(par);
 	if (!par->twoHeads) {
 		par->CRTCnumber = 0;
-		nvidia_probe_i2c_connector(par, 1, &edidA);
+		nvidia_probe_i2c_connector(info, 1, &edidA);
 		if (edidA && !fb_parse_edid(edidA, &var)) {
 			printk("nvidiafb: EDID found from BUS1\n");
 			monA = &monitorA;
@@ -488,14 +488,14 @@ void NVCommonSetup(struct fb_info *info)
 		oldhead = NV_RD32(par->PCRTC0, 0x00000860);
 		NV_WR32(par->PCRTC0, 0x00000860, oldhead | 0x00000010);
 
-		nvidia_probe_i2c_connector(par, 1, &edidA);
+		nvidia_probe_i2c_connector(info, 1, &edidA);
 		if (edidA && !fb_parse_edid(edidA, &var)) {
 			printk("nvidiafb: EDID found from BUS1\n");
 			monA = &monitorA;
 			fb_edid_to_monspecs(edidA, monA);
 		}
 
-		nvidia_probe_i2c_connector(par, 2, &edidB);
+		nvidia_probe_i2c_connector(info, 2, &edidB);
 		if (edidB && !fb_parse_edid(edidB, &var)) {
 			printk("nvidiafb: EDID found from BUS2\n");
 			monB = &monitorB;
_
